@@ -11,7 +11,7 @@ use Carp qw(croak carp);
 # LOAD CPAN MODULES
 #######################
 use Object::Tiny qw(id session);
-use Params::Validate qw(validate_with SCALAR OBJECT);
+use Params::Validate qw(validate_with :types);
 
 #######################
 # LOAD DIST MODULES
@@ -34,12 +34,14 @@ sub new {
                 type => OBJECT,
                 isa  => 'TMDB::Session',
             },
-            id => { type => SCALAR, },
+            id => {
+                type => SCALAR,
+            },
         },
     );
 
     my $self = $class->SUPER::new(%opts);
-    return $self;
+  return $self;
 } ## end sub new
 
 ## ====================
@@ -47,8 +49,33 @@ sub new {
 ## ====================
 sub info {
     my $self = shift;
-    return $self->session->talk( { method => 'collection/' . $self->id(), } );
-}
+  return $self->session->talk(
+        {
+            method => 'collection/' . $self->id(),
+            params => {
+                language => $self->session->lang
+                ? $self->session->lang
+                : undef,
+            },
+        }
+    );
+} ## end sub info
+
+## ====================
+## VERSION
+## ====================
+sub version {
+    my ($self) = @_;
+    my $response = $self->session->talk(
+        {
+            method       => 'collection/' . $self->id(),
+            want_headers => 1,
+        }
+    ) or return;
+    my $version = $response->{etag} || q();
+    $version =~ s{"}{}gx;
+  return $version;
+} ## end sub version
 
 ## ====================
 ## INFO HELPERS
@@ -64,17 +91,19 @@ sub ids { return shift->_parse_parts('id'); }
 # PRIVATE METHODS
 #######################
 
+
 sub _parse_parts {
     my $self  = shift;
     my $key   = shift;
-    my $parts = $self->info()->{parts} || [];
+    my $info  = $self->info();
+    my $parts = $info ? $info->{parts} : [];
     my @stuff;
     foreach my $part (@$parts) {
-        next unless $part->{$key};
+      next unless $part->{$key};
         push @stuff, $part->{$key};
-    }
-    return @stuff if wantarray;
-    return \@stuff;
+    } ## end foreach my $part (@$parts)
+  return @stuff if wantarray;
+  return \@stuff;
 } ## end sub _parse_parts
 
 #######################
